////////////////////////////////////////////////////
//
// Filename		: FlowControl.h
// Description		: Flow Control Class
//
// History
// -----------------------------------------------
// 20040129	Kei		First Create
//
////////////////////////////////////////////////////

#ifndef FLOWCONTROL_H
#define FLOWCONTROL_H

#include <time.h>

class FlowControl {
	protected:
		/**
        	*       @memo Description:
	        *               Internal data array
        	*/
		int	*stat, *statAll, defaultThreshold, defaultWindowSize;
		/**
        	*       @memo Description:
	        *               Internal timer
        	*/
		time_t	lastRequest, currentTime;

	protected:
		void clearBuffer();	
	public:
		/**
        	*       @memo Description:
        	*               Default constructor
        	*/
		FlowControl();
		
		/**
        	*       @memo Description:
        	*               Default destructor
        	*/
		virtual ~FlowControl();
		
		/**
	        *       @memo Description:
	        *               Initialize memory for storing system parameters
	        *       @exception
	        *               -
	        *       @return
	        *               -
	        *       @param key (input)
	        *               Default threshold and window size
	        *       @param flag (input)
	        *               -
	        */
		void init(int threshold, int windowSize);
		/**
	        *       @memo Description:
	        *               return default threshold
	        *       @exception
	        *               -
	        *       @return
	        *               default threshold
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int getThreshold() const;
		/**
	        *       @memo Description:
	        *               return default window size
	        *       @exception
	        *               -
	        *       @return
	        *               default window size
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int getWindow() const;
		/**
	        *       @memo Description:
	        *               Add flow counter and check the flow control status
	        *       @exception
	        *               -
	        *       @return
	        *               < 0  - Error (Object is not initialized)
	        *		= 0  - Under Flow Control
	        *		> 0  - Over Flow Control
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int add();
		/**
	        *       @memo Description:
	        *               Check the flow control status
	        *       @exception
	        *               -
	        *       @return
	        *               < 0  - Error (Object is not initialized)
	        *		= 0  - Under Flow Control
	        *		> 0  - Over Flow Control
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int check();
		/**
	        *       @memo Description:
	        *               Check the flow control status with provided threshold and window size
	        *       @exception
	        *               -
	        *       @return
	        *               < 0  - Error (Object is not initialized)
	        *		= 0  - Under Flow Control
	        *		> 0  - Over Flow Control
	        *       @param key (input)
	        *               threshold and window size
	        *       @param flag (input)
	        *               -
	        */
		int check(int threshold, int windowSize);
		/**
	        *       @memo Description:
	        *               Get the flow counter (under flow control)
	        *       @exception
	        *               -
	        *       @return
	        *               flow counter
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int get();
		/**
	        *       @memo Description:
	        *               Get the flow counter with provided window size (under flow control)
	        *       @exception
	        *               -
	        *       @return
	        *               flow counter
	        *       @param key (input)
	        *               window size
	        *       @param flag (input)
	        *               -
	        */
		int get(int windowSize);
		/**
	        *       @memo Description:
	        *               Get the flow counter (overall)
	        *       @exception
	        *               -
	        *       @return
	        *               flow counter
	        *       @param key (input)
	        *               -
	        *       @param flag (input)
	        *               -
	        */
		int getAll();
		/**
	        *       @memo Description:
	        *               Get the flow counter with provided window size (overall)
	        *       @exception
	        *               -
	        *       @return
	        *               flow counter
	        *       @param key (input)
	        *               window size
	        *       @param flag (input)
	        *               -
	        */
		int getAll(int windowSize);
};

#endif  
