/********************************************************************

	Filename	: IPC_Memory.h
	Description	: IPC Memory Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation

********************************************************************/

#ifndef IPC_MEMORY_H
#define IPC_MEMORY_H

#include "IPC_Object.h"
#ifdef WIN32
#include <windows.h>
#endif
class IPC_Memory : public IPC_Object {
protected:
	void*		shmaddr;
	size_t		shmsize;
#ifdef WIN32
	HANDLE	hMem;
#endif

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	IPC_Memory();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~IPC_Memory();

	/**
	*	@memo Description:
	*		Initialize and attach an inter-process communication (IPC) shared memory segment
	*	@exception
	*		GeneralErrorException - initialization failure
	*	@return
	*		Nil
	*	@param key (input)
	*		IPC unique key for shared memory segment, valid values from 0x00 to 0x7F
	*	@param flag (input)
	*		If (flag & IPC_CREAT) is true,
	*		shared memory segment will be created with permission given in other bits of the flag
	*/
	void init(int key, int flag);

	/**
	*	@memo Description:
	*		Initialize and attach a shared memory segment and set shared memory segment size
	*	@exception
	*		GeneralErrorException - initialization failure
	*	@return
	*		Nil
	*	@param key (input)
	*		IPC unique key for shared memory segment, valid values from 0x00 to 0x7F
	*	@param flag (input)
	*		If (flag & IPC_CREAT) is true,
	*		shared memory segment will be created with permission given in other bits of the flag
	*	@param size (input)
	*		Shared memory size
	*/
	void init(int key, int flag, size_t size);

	/**
	*	@memo Description:
	*		Detach and destroy a shared memory segment
	*	@exception
	*		GeneralErrorException - destroy failure
	*	@return
	*		Nil
	*/
	void destroy();

	/**
	*	@memo Description:
	*		Set shared memory size before creation or attaching the shared memory segment
	*	@exception
	*		GeneralErrorException - invalid size
	*	@return
	*		Nil
	*	@param size (input)
	*		Shared memory size
	*/
	void set_SharedMemorySize(size_t size);

	/**
	*	@memo Description:
	*		Get shared memory size after creation or attaching the shared memory segment
	*	@exception
	*		None
	*	@return
	*		Shared memory size
	*/
	size_t get_SharedMemorySize() const;

	/**
	*	@memo Description:
	*		Get shared memory address after attaching the shared memory segment
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param shmaddr (output)
	*		Pointer to shared memory segment to be obtained
	*/
	void get_SharedMemoryAddress(void **shmaddr) const;
};

#endif		/* IPC_MEMORY_H */
