/********************************************************************

	Filename	: IPC_Object.h
	Description	: IPC Object Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011023	Michael Chen: Add new_create_flag

********************************************************************/

#ifndef IPC_OBJECT_H
#define IPC_OBJECT_H

#include <sys/types.h>
#ifdef WIN32
typedef	int	key_t;
#else
#include <sys/ipc.h>
#endif

#define PERMS		0660
#define MY_KEY_FILE	"./projkey"

class IPC_Object {
protected:
	/**
	*	@memo Description:
	*		Inter-process communication (IPC) object identifier
	*/
	int		id;

	/**
	*	@memo Description:
	*		Flag to indicate whether the IPC object is newly created
	*/
	int		newCreate;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	IPC_Object();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~IPC_Object();

	/**
	*	@memo Description:
	*		Check if the IPC object is valid (already been initialized)
	*	@exception
	*		None
	*	@return
	*		1 if valid, 0 if not valid
	*/
	int isValid() const;

	/**
	*	@memo Description:
	*		Get the IPC object identifier
	*	@exception
	*		None
	*	@return
	*		IPC object identifier
	*/
	int get_id() const;

	/**
	*	@memo Description:
	*		Get the key value with .projadm from a key
	*	@exception
	*		GeneralErrorException - invalid key or .projadm not found
	*	@return
	*		Key value
	*	@param key (input)
	*		Input key
	*/
	key_t get_key(int key) const;

	/**
	*	@memo Description:
	*		Initialize and attach an IPC object.
	*		It must be implemented for different types of IPC objects
	*	@exception
	*		GeneralErrorException - initialization failure
	*	@return
	*		Nil
	*	@param key (input)
	*		IPC unique key for an object, valid values from 0x00 to 0x7F
	*	@param flag (input)
	*		If (flag & IPC_CREAT) is true,
	*		shared memory segment will be created with permission given in other bits of the flag
	*/
	virtual void init(int key, int flag) = 0;

	/**
	*	@memo Description:
	*		Detach and destroy an IPC object.
	*		It must be implemented for different types of IPC objects
	*	@exception
	*		GeneralErrorException - destroy failure
	*	@return
	*		Nil
	*/
	virtual void destroy() = 0;

	/**
	*	@memo Description:
	*		Check if the IPC object is a newly created object
	*	@exception
	*		None
	*	@return
	*		1 if newly created, 0 otherwise
	*/
	int isNewCreate() const;
};

#endif		/* IPC_OBJECT_H */
