/********************************************************************

	Filename	: IPC_Queue.h
	Description	: IPC Queue Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011101	Michael Chen: Add last_recv_mtype
	20040213	Michael Chen: Add statistics
	2004

********************************************************************/

#ifndef IPC_QUEUE_H
#define IPC_QUEUE_H

#include "IPC_Object.h"

class IPC_Queue : public IPC_Object {
private:
	long		last_recv_mtype;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	IPC_Queue();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~IPC_Queue();

	/**
	*	@memo Description:
	*		Initialize and attach an inter-process communication (IPC) message queue
	*	@exception
	*		GeneralErrorException - initialization failure
	*	@return
	*		Nil
	*	@param key (input)
	*		IPC unique key for message queue, valid values from 0x00 to 0x7F
	*	@param flag (input)
	*		If (flag & IPC_CREAT) is true,
	*		message queue will be created with permission given in other bits of the flag
	*/
	void init(int key, int flag);

	/**
	*	@memo Description:
	*		Detach and destroy an IPC message queue
	*	@exception
	*		GeneralErrorException - destroy failure
	*	@return
	*		Nil
	*/
	void destroy();

	/**
	*	@memo Description:
	*		Send a message to the message queue with no wait
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		0 if successful
	*	@param msg (input)
	*		Pointer to message to be sent
	*	@param len (input)
	*		Message size to be sent
	*	@param mtype (input)
	*		Message type
	*/
	int send(void *msg, size_t len, long mtype);

	/**
	*	@memo Description:
	*		Send a message to the message queue with no wait and with retry
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		0 if successful
	*	@param msg (input)
	*		Pointer to message to be sent
	*	@param len (input)
	*		Message size to be sent
	*	@param mtype (input)
	*		Message type
	*	@param retry (input)
	*		Number of retries
	*/
	int send(void *msg, size_t len, long mtype, int retry);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with specific mtype with no wait
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (input)
	*		Message type of the message to be received
	*/
	int recv_nowait(void *msg, size_t& len, long mtype);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with specific mtype.
	*		Function returns until there is a message.
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (input)
	*		Message type of the message to be received
	*/
	int recv(void *msg, size_t& len, long mtype);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with specific mtype.
	*		Function returns until there is a message or timeout is reached.
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (input)
	*		Message type of the message to be received
	*	@param timeout (input)
	*		Timeout value (in seconds) for waiting message
	*/
	int recv(void *msg, size_t& len, long mtype, time_t timeout);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with all mtype with no wait
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (output)
	*		Message type of the received message to be obtained
	*/
	int recvAll_nowait(void *msg, size_t& len, long *mtype);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with all mtype.
	*		Function returns until there is a message.
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (output)
	*		Message type of the received message to be obtained
	*/
	int recvAll(void *msg, size_t& len, long *mtype);

	/**
	*	@memo Description:
	*		Receive a message from the message queue with all mtype.
	*		Function returns until there is a message or timeout is reached.
	*	@exception
	*		None
	*	@return
	*		-1 if failed,
	*		return number of bytes received if successful
	*	@param msg (output)
	*		Pointer to message to be received
	*	@param len (output)
	*		Received Message size
	*	@param mtype (input)
	*		Message type of the message to be received
	*	@param mtype (output)
	*		Message type of the received message to be obtained
	*/
	int recvAll(void *msg, size_t& len, long *mtype, time_t timeout);

	/**
	*	@memo Description:
	*		Get the message type of the last received message
	*	@exception
	*		None
	*	@return
	*		Message type of the last received message
	*/
	long get_last_recv_mtype();

	/**
	*	@memo Description:
	*		Get the statistics structure of the message queue
	*	@exception
	*		GeneralErrorException - get failure
	*	@param msg (output)
	*		Pointer to the statistics structure
	*	@return
	*		Nil
	*/
	void get_stat(struct msqid_ds *buf);

	/**
	*	@memo Description:
	*		Get the current number of bytes on the message queue
	*	@exception
	*		GeneralErrorException - get failure
	*	@return
	*		Current number of bytes on the message queue
	*/
	int get_stat_msg_cbytes();

	/**
	*	@memo Description:
	*		Get the number of messages on the message queue
	*	@exception
	*		GeneralErrorException - get failure
	*	@return
	*		Number of messages on the message queue
	*/
	int get_stat_msg_qnum();

	/**
	*	@memo Description:
	*		Get the maximum number of bytes on the message queue
	*	@exception
	*		GeneralErrorException - get failure
	*	@return
	*		Maximum number of bytes on the message queue
	*/
	int get_stat_msg_qbytes();

};

#endif		/* IPC_QUEUE_H */
