/********************************************************************

	Filename	: LogFile.h
	Description	: LOG File Class header

	History:
	------------------------------------------------------------
	20011128	Michael Chen
			- Creation from proutil.c
	20030915	Michael Chen
			- Added mutex to support multi-thread
	20040213	Michael Chen (in cpplib v1.22)
			- Migrate to pthread_mutex instead of mutex
			- Add function get_file_size()
	20040217	Michael Chen (in cpplib v1.22)
			- Add function switch_log()
			- Add function setLogFlag()

********************************************************************/
#ifndef LOGFILE_H
#define LOGFILE_H

#include <stdio.h>

#if !defined(WIN32)
	#include <pthread.h>
#endif

#define MAX_MSG_SIZE		1024
#define MAX_LOGFILE_MSG		500000
#define MAX_LOG_BUFFER_CNT	100

#define LOG_FILE_LEVEL	50
#define LOG_PRINT_LEVEL	70

class LogFile {
private:
	FILE *logfile;
	char logfile_name[64];
	char logfile_name_bak[64];
	int max_logfile_msg;
	unsigned int	flush_counter;
	unsigned int	max_flush_counter;
	unsigned int	log_flag;
#if !defined(WIN32)
	pthread_mutex_t mp;
#endif

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	LogFile();

	/**
	*	@memo Description:
	*		Default constructor with initializing the filename of log file and log backup file
	*	@param logfilename (input)
	*		Current log file name
	*	@param logfilebak (input)
	*		Log backup file name
	*/
	LogFile(const char *logfilename, const char *logfilebak);

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~LogFile();

	/**
	*	@memo Description:
	*		Set maximum size of the log file.
	*		When the log file size reach the maximum size,
	*		the log file will be backup to the log backup file.
	*		Default size is 500000 bytes.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param max_size (input)
	*		Maximum log file size to be set
	*/
	void setMaxLogSize(int max_size);

	/**
	*	@memo Description:
	*		Set log buffer size (number of log message)
	*		before flushing to the file.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param cnt (input)
	*		Number of log messages before flush
	*/
	void setLogBufferSize(int cnt);

	/**
	*	@memo Description:
	*		Set log flag
	*		0 = normal
	*		1 = switch file daily
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param flag (input)
	*		Log flag
	*/
	void setLogFlag(int flag);

	/**
	/**
	*	@memo Description:
	*		Initializing the filename of log file and log backup file
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		-1 if failed (e.g. file open error)
	*	@param logfilename (input)
	*		Current log file name
	*	@param logfilebak (input)
	*		Log backup file name
	*/
	int init_log(const char *logfilename, const char *logfilebak);

	/**
	*	@memo Description:
	*		Log message content into the log file
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param str (input)
	*		Message string to be logged into the file
	*/
	void log_msg(const char *str);

	/**
	*	@memo Description:
	*		Log message content into the log file with log level specified.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param str (input)
	*		Message string to be logged into the file
	*	@param output_level (input)
	*		Output level.
	*		If output_level < 50, there will be no output of message content.
	*		If output_level >= 50, message will be logged into the file.
	*		If output_level >= 70, message will also be printed to the standard output.
	*/
	void log_msg(const char *str, int output_level);

	/**
	*	@memo Description:
	*		Switch and backup the log file
	*	@exception
	*		None
	*	@return
	*		Nil
	*/
	void switch_log();

	/**
	*	@memo Description:
	*		Close the log file
	*	@exception
	*		None
	*	@return
	*		Nil
	*/
	void close_log();

	/**
	*	@memo Description:
	*		Obtain the log file size
	*	@exception
	*		None
	*	@return
	*		Nil
	*/
	int get_file_size();
};

#endif
