/***********************************************************************

	Filename	: Semaphore.h
	Description	: Semaphore Class header file

	History:
	------------------------------------------------------------
	20011207	Michael Chen: First Creation

***********************************************************************/

#ifndef SEMAPHORE_H
#define SEMAPHORE_H

#ifndef __linux__

#include <synch.h>

class Semaphore {
private:
	sema_t		*sp;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	Semaphore();

	/**
	*	@memo Description:
	*		Constructor with semaphore memory initialization
	*	@param ptr (input)
	*		Semaphore pointer to be initialized
	*/
	Semaphore(sema_t *ptr);

	/**
	*	@memo Description:
	*		Default destructor (without destroying the semaphore)
	*/
	virtual ~Semaphore();

	/**
	*	@memo Description:
	*		Initialize semaphore memory by assigning the semaphore address
	*	@exception
	*		None
	*	@return
	*		0 if valid pointer,
	*		-1 otherwise
	*	@param ptr (input)
	*		Semaphore pointer to be initialized
	*/
	int init_mem(sema_t *ptr);

	/**
	*	@memo Description:
	*		Initialize semaphore memory by creating semaphore in local memory
	*	@exception
	*		None
	*	@return
	*		0
	*/
	int init_mem();

	/**
	*	@memo Description:
	*		Initialize a semaphore after initialized semaphore memory with types
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*	@param count (input)
	*		Semaphore count
	*	@param type (input)
	*		Type of semaphore.
	*		USYNC_THREAD for synchronization among threads only.
	*		USYNC_PROCESS for synchronization among threads and processes.
	*/
	int init(int count, int type);

	/**
	*	@memo Description:
	*		Initialize a semaphore with USYNC_PROCESS type after initialized semaphore memory
	*	@exception
	*		None
	*	@param count (input)
	*		Semaphore count
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int init(int count);

	/**
	*	@memo Description:
	*		Destroy a semaphore
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int destroy();

	/**
	*	@memo Description:
	*		Try to decrement the semaphore count.
	*		Function return immediately no matter the semaphore count can be decremented successfully or not
	*	@exception
	*		None
	*	@return
	*		0 if successfully decremented the semaphore count.
	*		EBUSY if the semaphore count is already zero,
	*		errno otherwise
	*/
	int trywait();

	/**
	*	@memo Description:
	*		Decrement the semaphore count.
	*		If the semaphore count is already zero,
	*		it will wait until the semaphore count is greater than zero.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int wait();

	/**
	*	@memo Description:
	*		Increment the semaphore count.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int post();
};

#endif		/* __linux__ */

#endif		/* SEMAPHORE_H */
