/********************************************************************

	Filename	: SysUtil.h
	Description	: System Utility Header

	History:
	------------------------------------------------------------
	20010701	Michael Chen
			- First Creation
	20020118	Michael Chen
			- Added ct_fopen_r(), ct_fgets(), ct_fclose()
	20040213	Michael Chen (in cpplib v1.22)
			- Added CURR_RPT_ENV
	20040226	Michael Chen (in cpplib v1.22)
			- Added get_host_ip()

********************************************************************/

#ifndef SYSUTIL_H
#define SYSUTIL_H

#define ENVNAME		"BASEDIR"
#define CFG_ENV		"CONFDIR"
#define BIN_ENV		"BINDIR"
#define RPT_ENV		"LOGDIR"
#define RPT_ENV2	"LOGDIR2"
#define RPT_ENV3	"RPTDIR"
#define CURR_RPT_ENV	"CURRLOGDIR"

/**
*	@memo Description:
*		Sleep for duration in milli-seconds
*	@exception
*		None
*	@return
*		Nil
*	@param interval (input)
*		Duration in milli-seconds
*/
void msleep(long interval);

/**
*	@memo Description:
*		Get a random number
*	@exception
*		None
*	@return
*		Random number from 0 to range-1
*	@param range (input)
*		Range of the random number.
*		Generated random number value will be between 0 and range-1 inclusively.
*/
int getRandomInteger(int range);

/**
*	@memo Description:
*		Get a configuration full path of the filename
*	@exception
*		None
*	@return
*		Pointer to character array to the configuration full path to be obtained
*	@param path (output)
*		Pointer to character array to the configuration full path to be obtained
*	@param filename (input)
*		Filename
*/
char *getcfgpath(char *path, const char *filename);

/**
*	@memo Description:
*		Get the machine ID
*	@exception
*		None
*	@return
*		1 if local machine name is ended with odd digit or non-'B' alphabet
*		2 if local machine name is ended with even digit or 'B' alphabet
*/
int getmachinename();

/**
*	@memo Description:
*		Get a value from the string.
*		If the string is started by 0x, it is treated as a hexadecimal number.
*		If the string is started by 0, it is treated as a octal number.
*		Otherwise, it is treated as a decimal number.
*	@exception
*		None
*	@return
*		Value of the string
*/
long str2long(char *str);

/**
*	@memo Description:
*		Open and read a configuration file within a tag inside the branckets '[' and ']'
*	@exception
*		None
*	@return
*		NULL if file open error or tag not found,
*		file pointer otherwise
*	@param filename (input)
*		Configuration file name
*	@param tag_name (input)
*		Tag name, excluding '[' and ']' characters
*/
FILE* ct_fopen_r(const char *filename, const char *tag_name);

/**
*	@memo Description:
*		Get a line of configuration file within a tag
*	@exception
*		None
*	@return
*		NULL if end of file reached or end of tag reached (begin of another tag),
*		pointer to character array of the line read if otherwise
*	@param s (output)
*		Pointer to character array of the line to be read
*	@param n (input)
*		Maximum bytes to be read
*	@param stream (input)
*		File pointer
*/
char* ct_fgets(char *s, int n, FILE *stream);

/**
*	@memo Description:
*		Close a configuration file
*	@exception
*		None
*	@return
*		0 if successful,
*		EOF otherwise
*	@param stream (input)
*		File pointer
*/
int ct_fclose(FILE *stream);

/**
*	@memo Description:
*		Get a token value from a string,
*		e.g. token = "DEBUG=100" and title = "DEBUG", then return string will be "100"
*	@exception
*		None
*	@return
*		Pointer to character array if token matched,
*		NULL otherwise
*	@param token (input)
*		Token string
*	@param title (input)
*		Token title
*/
char* get_token_value(char *token, char* title);

/**
*	@memo Description:
*		Get a token value from a string,
*		e.g. token = "DEBUG=100,PORT=20001;", title = "DEBUG", delim1 = "=", delim2 = ","
*		(title must be the beginning of token string),
*		then return string will be "100"
*	@exception
*		None
*	@return
*		Pointer to character array if token matched,
*		NULL otherwise
*	@param token (input)
*		Token string
*	@param title (input)
*		Token title
*	@param delim1 (input)
*		First delimitor
*	@param delim2 (input)
*		Second delimitor
*	@param value (output)
*		Output token value
*/
char* get_token_value(char *token, char *title, char *delim1, char *delim2, char *value);

/**
*	@memo Description:
*		Read configuration file with function provided
*	@exception
*		None
*	@return
*		Number of parameter read
*	@param filename (input)
*		Configuration file name
*	@param tag (input)
*		Configuration tag value, e.g. "[APP1_STR]"
*	@param flag (input)
*		If use $CONFDIR, set flag to non zero.
*		If does not use $CONFDIR, set flag to zero.
*	@param func (input)
*		Function to be passed to obtain parameter.
*		Function prototype (char*, void*, int, int).
*		First parameter will be the line read from the configuration file.
*		Second parameter will be the pointer to parameter structure.
*		Third parameter will be the line number.
*		Fourth parameter will be the function flag.
*	@param ptr (input)
*		Pointer of parameter structure to obtain the parameter.
*		This will be passed as the second parameter of the func().
*	@param func_flag (input)
*		Function flag to be passed as the fourth parameter of the func().
*/
int read_config_file(char *filename, char *tag, int flag, int (func)(char*, void*, int, int), void* ptr, int func_flag);

/**
*	@memo Description:
*		Read a parameter value in a configuration file.
*		e.g. If the line "DEBUG=100" exists in the file <filename>
*		under the tag <tag>, e.g. [APP1_SW],
*		and name = "DEBUG", then return string will be "100"
*	@exception
*		None
*	@return
*		Pointer to character array if token matched,
*		NULL otherwise
*	@param filename (input)
*		Configuration file name
*	@param tag (input)
*		Configuration tag
*	@param name (input)
*		Token title
*/
char* get_config_param(char *filename, char *tag, char *name);

/**
*	@memo Description:
*		Enhanced version of signal system function
*	@exception
*		None
*	@return
*		Signal's previous disposition
*	@param signum (input)
*		Signal ID
*	@param disp (input)
*		New signal disposition
*/
void (*signal2(int signum, void (*disp)(int)))(int);

/**
*	@memo Description:
*		Obtain the IP from the Hostname
*	@exception
*		None
*	@return
*		Host IP
*	@param hostname (input)
*		Hostname, e.g. "sun74" or "www.ithlgroup.com"
*	@param host_ip (output)
*		Host IP (can be NULL)
*/
char *get_host_ip(char *hostname, char *host_ip);

#endif /* SYSUTIL_H */
