/********************************************************************

	Filename	: TCPClient.h
	Description	: TCP Client Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011120	Michael Chen: Added readServerInfoTagConfig
	20020601	Michael Chen: Added connectAllServersThread() and connectServerThread()

********************************************************************/

#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include "TCPSocket.h"

class TCPClient {
protected:
	/***
	*	constant values
	*/
	enum {
		maxServers = 256,
		maxConnection = maxServers,
		maxIPlen = 20,
		maxFeature = 40
	};

	/**
	*	@memo Description:
	*		TCP Socket array
	*/
	TCPSocket	*tcpsock;

private:
	char		server_ip[maxServers][maxIPlen+1];
	unsigned short	server_port[maxServers];
	unsigned short	client_port[maxConnection];
	char		server_type[maxServers][maxFeature+1];

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	TCPClient();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~TCPClient();

	/**
	*	@memo Description:
	*		Read server configuration file without any tag name,
	*		formatted by each line having 5 fields (server id, server ip, server port, client port, server type),
	*		where client port and server type are both optional fields.
	*		Maximum length of server type is 40 characters.
	*		Client port is 0 or optional means bind client port with any port address.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*/
	virtual TCPClient& readServerConfig(const char *filename);

	/**
	*	@memo Description:
	*		Read server configuration file under a tag "[SERVER_INFO]",
	*		formatted by each line having 5 fields (server id, server ip, server port, client port, server type),
	*		where client port and server type are both optional fields.
	*		Maximum length of server type is 40 characters.
	*		Client port is 0 or optional means bind client port with any port address.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error or tag not found
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*/
	virtual TCPClient& readServerInfoTagConfig(const char *filename);

	/**
	*	@memo Description:
	*		Read server configuration file under a dynamic tag, e.g. tag "ABC" means under tag "[ABC]" in the file,
	*		content formatted by each line having 5 fields (server id, server ip, server port, client port, server type),
	*		where client port and server type are both optional fields.
	*		Maximum length of server type is 40 characters.
	*		Client port is 0 or optional means bind client port with any port address.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error or tag not found
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*	@param tag_name (input)
	*		Tag name
	*/
	virtual TCPClient& readServerInfoTagConfig(const char *filename, const char *tag_name);

	/**
	*	@memo Description:
	*		Set server configuration for one server ID,
	*		with 4 fields (server id, server ip, server port, client port),
	*		and server type will be empty string.
	*		Client port is 0 or optional means bind client port with any port address.
	*	@exception
	*		Nil
	*	@return
	*		0 if valid server id,
	*		-1 otherwise.
	*	@param id (input)
	*		Server ID
	*	@param s_ip (input)
	*		Server IP (Hostname is also acceptable)
	*	@param s_port (input)
	*		Server port
	*	@param c_port (input)
	*		Client port
	*/
	int setServerConfig(int id, char *s_ip, unsigned short s_port, unsigned short c_port);

	/**
	*	@memo Description:
	*		Set server configuration for one server ID,
	*		with 5 fields (server id, server ip, server port, client port, server type).
	*		Maximum length of server type is 40 characters.
	*		Client port is 0 or optional means bind client port with any port address.
	*	@exception
	*		Nil
	*	@return
	*		0 if valid server id,
	*		-1 otherwise.
	*	@param id (input)
	*		Server ID
	*	@param s_ip (input)
	*		Server IP (Hostname is also acceptable)
	*	@param s_port (input)
	*		Server port
	*	@param c_port (input)
	*		Client port
	*	@param s_type (input)
	*		Server type
	*/
	int setServerConfig(int id, char *s_ip, unsigned short s_port, unsigned short c_port, char *s_type);

	/**
	*	@memo Description:
	*		Clear server configuration for one server ID,
	*	@exception
	*		Nil
	*	@return
	*		0 if valid server id,
	*		-1 otherwise.
	*	@param id (input)
	*		Server ID
	*/
	int clearServerConfig(int id);

	/**
	*	@memo Description:
	*		Get server type of a server ID
	*	@exception
	*		Nil
	*	@return
	*		0 if valid server id,
	*		-1 otherwise.
	*	@param id (input)
	*		Server ID
	*	@param s_type (output)
	*		Server type to be obtained
	*/
	int getServerType(int id, char *s_type);

	/**
	*	@memo Description:
	*		Check if the server ID's configuration has been set
	*	@exception
	*		Nil
	*	@return
	*		0 if invalid server id or configuration not set,
	*		1 if valid server id and configuration set.
	*	@param id (input)
	*		Server ID
	*/
	int isValidServerConfiguration(int id) const;

	/**
	*	@memo Description:
	*		Check if the server ID's connection is established
	*	@exception
	*		Nil
	*	@return
	*		0 if invalid server id or connection not established (disconnected),
	*		1 if valid server id and connection established (connected).
	*	@param id (input)
	*		Server ID
	*/
	int isValidServerConnection(int id) const;

	/**
	*	@memo Description:
	*		Get number of established connections
	*	@exception
	*		Nil
	*	@return
	*		Number of established connections
	*/
	int getNumActiveServers() const;

	/**
	*	@memo Description:
	*		Connect all servers.
	*		This function checks if any server's configuration has been set but connection not established,
	*		and then perform connection on that server.
	*		This function is blocking and should return after all connection action returns.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		0 if no error during connection or no connection action needed,
	*		-1 if error detected during connection.
	*/
	virtual int connectAllServers();

	/**
	*	@memo Description:
	*		Connect a server (create socket, bind and connect server).
	*		This function is blocking and should return after the connection action returns.
	*		This function can be overrided.
	*	@exception
	*		GeneralErrorException - invalid server ID
	*	@return
	*		0 if no error during connection,
	*		-1 if error detected during connection or connection already established.
	*/
	virtual int connectServer(int id);

	/**
	*	@memo Description:
	*		Connect all servers without blocking (using thread).
	*		This function checks if any server's configuration has been set but connection not established,
	*		and then perform connection on that server.
	*		This function is non-blocking and should return immediately no matter the connections are successful or not.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		0 if no error during connection or no connection action needed,
	*/
	virtual int connectAllServersThread();

	/**
	*	@memo Description:
	*		Connect a server (create socket, bind and connect server) without blocking (using thread).
	*		This function is blocking and should return after the connection action returns.
	*		This function can be overrided.
	*	@exception
	*		GeneralErrorException - invalid server ID
	*	@return
	*		0 if no error during connection,
	*		-1 if error detected during connection or connection already established.
	*/
	virtual int connectServerThread(int id);

	/**
	*	@memo Description:
	*		Close all active server connections.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		Always 0.
	*/
	virtual int disconnectAllServers();

	/**
	*	@memo Description:
	*		Close a server connection.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		0 if successfully disconnected,
	*		-1 if server connection already disconnected.
	*	@param id (input)
	*		Server ID
	*/
	virtual int disconnectServer(int id);

	/**
	*	@memo Description:
	*		Poll all server event (e.g. incoming data or disconection).
	*		When there is event on a server ID,
	*		virtual function processServerEvent() will be called automatically.
	*		The function processServerEvent() should handle the event.
	*		If error detected, processServerEvent() should return -1 and pollServerEvent() will perform disconnection automatically.
	*	@exception
	*		Nil
	*	@return
	*		0 if polling timeout,
	*		-1 if polling error,
	*		otherwise, return number of events.
	*	@param timeout (input)
	*		Timeout value in milli-seconds.
	*/
	int pollServerEvent(int timeout);

	/**
	*	@memo Description:
	*		Read data from a connection
	*	@exception
	*		GeneralErrorException - invalid server ID
	*	@return
	*		0 if disconnected by remote side.
	*		-1 if error,
	*		otherwise, return number of bytes read.
	*	@param id (input)
	*		Server ID
	*	@param buffer (output)
	*		Buffer of data to be read
	*	@param len (input)
	*		Maximum length of buffer can be read
	*/
	int readServer(int id, unsigned char *buffer, int len);

	/**
	*	@memo Description:
	*		Send data to a connection
	*	@exception
	*		GeneralErrorException - invalid server ID
	*	@return
	*		0 if disconnected by remote side.
	*		-1 if error,
	*		otherwise, return number of bytes sent.
	*	@param id (input)
	*		Server ID
	*	@param buffer (output)
	*		Buffer of data to be sent
	*	@param len (input)
	*		Length of buffer to be sent
	*/
	int sendServer(int id, unsigned char *buffer, int len);

	/**
	*       @memo Description:
	*               Process server event, e.g. incoming message or disconnection from server.
	*               It is called automatically when there is a server event when pollServerEvent() is called.
	*               It must be overrided and user should call readServer() to read message
	*               and return -1 when readServer() returns <= 0.
	*       @exception
	*               None
	*       @return
	*               -1 when there is error in event handling, e.g. disconnection case,
	*               0 when successful
	*       @param id (input)
	*               Server ID
	*/
	virtual int processServerEvent(int id) = 0;
};

#endif /* TCPCLIENT_H */
