/********************************************************************

	Filename	: Logger.h
	Description	: Logger Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20020722	Ko Chun Yee: Add writeHeader(), writeTrailer()

********************************************************************/

#include <stdio.h>
#ifndef LOGGER_H
#define LOGGER_H

#include <stdio.h>
#include "IPC_Queue.h"

#define MAX_LOG_NAME_LEN	30

#define LOGGER_DAILY		0
#define LOGGER_HOURLY		1

class Logger {
private:	// variables
	int		backup_enable;
	int		mtype;
	IPC_Queue	logq;
	FILE		*fp[2];
	int		quit;

protected:	// variables
	/**
	*	@memo Description:
	*		Log file name, e.g. SYSTEM.LOG
	*/
	char		syslog[MAX_LOG_NAME_LEN+1];

	/**
	*	@memo Description:
	*		Backup log file extension, e.g. SYS for YYYYMMDD.SYS
	*/
	char		logext[MAX_LOG_NAME_LEN+1];

	/**
	*	@memo Description:
	*		Backup log directory name, e.g. SYS for $LOGDIR/SYS
	*/
	char		logdir[MAX_LOG_NAME_LEN+1];

public:		// functions
	/**
	*	@memo Description:
	*		Default constructor
	*/
	Logger();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~Logger();

	/**
	*	@memo Description:
	*		Parse logger's command arguments
	*	@exception
	*		None
	*	@return
	*		0
	*	@param argc (input)
	*		Argument count.  Set backup enable when argc > 1.
	*	@param argv (input)
	*		Array of character pointers to the arguments.
	*/
	int parseCommandArgument(int argc, char **argv);

	/**
	*	@memo Description:
	*		Set message type for receiving log message
	*	@exception
	*		None
	*	@return
	*		0
	*	@param type (input)
	*		Message type to be received from IPC message queue.
	*		If (type == 0), the first message on the queue is received.
	*		If (type > 0), the first message of the same type is received.
	*		If (type < 0), the first message of the lowest type that is <= abs(type) is received.
	*/
	int set_mtype(int type);

	/**
	*	@memo Description:
	*		Set name of the log file
	*	@exception
	*		None
	*	@return
	*		0
	*	@param filename (input)
	*		Filename of the log file to be set, e.g. SYSTEM.LOG
	*/
	int setLogFileName(const char *filename);

	/**
	*	@memo Description:
	*		Set file extension of the backup log file
	*	@exception
	*		None
	*	@return
	*		0
	*	@param ext (input)
	*		Extension of the backup log file to be set, e.g. SYS for filename YYYYMMDD.SYS
	*/
	int setLogExt(const char *ext);
	
	/**
	*	@memo Description:
	*		Set directory of the backup log file
	*	@exception
	*		None
	*	@return
	*		0
	*	@param dir (input)
	*		Directory of the backup log file to be set, e.g. SYS for directory $LOGDIR/SYS
	*/
	int setLogDir(const char *dir);
	
	/**
	*	@memo Description:
	*		Open message queue and log file in daily mode,
	*		wait for message and call write_LOG() internally
	*	@exception
	*		GeneralErrorException - Initialize message queue failed
	*	@return
	*		0 if OK,
	*		-1 if failed
	*/
	int init_logger();
	
	/**
	*	@memo Description:
	*		Open message queue and log file
	*		wait for message and call write_LOG() internally
	*	@exception
	*		GeneralErrorException - Initialize message queue failed
	*	@return
	*		0 if OK,
	*		-1 if failed.
	*	@param flag (input)
	*		When set to LOGGER_DAILY (0), log will be backup daily as YYYYMMDD.<ext>.
	*		When set to LOGGER_HOURLY (1), log will be backup hourly as YYYYMMDDhh.<ext>.
	*	@see
	*		init_logger();
	*/
	int init_logger(int flag);

	/**
	*	@memo Description:
	*		Ignore signal mask for SIGHUP, SIGINT, SIGQUIT, SIGTERM, SIGUSR1 and SIGUSR2
	*	@exception
	*		None
	*	@return
	*		Self reference
	*/
	Logger& set_signal();

private:	// functions
	/**
	*	@memo Description:
	*		Generate the backup file full path
	*	@exception
	*		None
	*	@return
	*		Pointer to character arrays of the new path
	*	@param env (input)
	*		Environment variable of the log directory
	*	@param path (output)
	*		Pointer to character arrays of the new path 
	*	@param name (input)
	*		Backup log directory name
	*	@param filename (input)
	*		Backup log file name
	*/
	char* genpath(char *env, char *path, const char *name, const char *filename);

	/**
	*	@memo Description:
	*		Generate the backup file full path
	*	@exception
	*		None
	*	@return
	*		Pointer to character arrays of the new path
	*	@param env (input)
	*		Environment variable of the log directory
	*	@param path (output)
	*		Pointer to character arrays of the new path 
	*	@param name (input)
	*		Backup log directory name
	*	@param filename (input)
	*		Backup log file name
	*/
	char* genpath2(char *env, char *path, const char *name, const char *filename);

public:		// functions
	/**
	*	@memo Description:
	*		Write information in Log.  Invoke automatically if message received.
	*		This function must be implemented in the derived class.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param f (input)
	*		File pointer of the opened log file
	*	@param msg (input)
	*		Pointer to the log data message which contains information to be logged into file
	*/
	virtual void write_LOG(FILE *f, unsigned char *msg) = 0;

	/**
	*	@memo Description:
	*		Write header information in Log.  Invoke automatically when open a new file.
	*		This function can be overrided in the derived class.
	*		If not overrided, original function will write nothing in the header.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param f (input)
	*		File pointer of the opened log file
	*/
	virtual void writeHeader(FILE *f);

	/**
	*	@memo Description:
	*		Write trailer information in Log.  Invoke automatically before file closing.
	*		This function can be overrided in the derived class.
	*		If not overrided, original function will write nothing in the trailer.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param f (input)
	*		File pointer of the opened log file
	*/
	virtual void writeTrailer(FILE *f);
};

#endif	/* LOGGER_H */
