/********************************************************************

	Filename	: Mutex.h
	Description	: Mutex Class header file

	History:
	------------------------------------------------------------
	20011103	Michael Chen: First Creation
	20011206	Michael Chen: Add init(int type)

********************************************************************/

#ifndef MUTEX_H
#define MUTEX_H

#include <thread.h>

class Mutex {
private:
	mutex_t		*mp;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	Mutex();

	/**
	*	@memo Description:
	*		Constructor with mutex memory initialization
	*	@param ptr (input)
	*		Mutex pointer to be initialized
	*/
	Mutex(mutex_t *ptr);

	/**
	*	@memo Description:
	*		Default destructor (without destroying the mutex)
	*/
	~Mutex();

	/**
	*	@memo Description:
	*		Initialize mutex memory by assigning the mutex address
	*	@exception
	*		None
	*	@return
	*		0 if valid pointer,
	*		-1 otherwise
	*	@param ptr (input)
	*		Mutex pointer to be initialized
	*/
	int init_mem(mutex_t *ptr);

	/**
	*	@memo Description:
	*		Initialize mutex memory by creating mutex in local memory
	*	@exception
	*		None
	*	@return
	*		0
	*/
	int init_mem();

	/**
	*	@memo Description:
	*		Initialize a mutex after initialized mutex memory with types
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*	@param type (input)
	*		Type of mutex.
	*		USYNC_THREAD for synchronization among threads only.
	*		USYNC_PROCESS for synchronization among threads and processes.
	*		USYNC_PROCESS_ROBUST (Solaris 2.7 onwards)
	*		for synchronization among threads and processes robustly
	*		to handle case that mutex owner is dead.
	*/
	int init(int type);

	/**
	*	@memo Description:
	*		Initialize a mutex with USYNC_PROCESS type after initialized mutex memory
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int init();

	/**
	*	@memo Description:
	*		Destroy a mutex
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int destroy();

	/**
	*	@memo Description:
	*		Try to lock a mutex.
	*		Function return immediately no matter the mutex can be locked successfully or not
	*	@exception
	*		None
	*	@return
	*		0 if successfully locked a mutex,
	*		EBUSY if the mutex is already locked by another process / thread,
	*		errno otherwise
	*/
	int trylock();

	/**
	*	@memo Description:
	*		Lock a mutex.
	*		If the mutex is being locked by another process,
	*		it will wait until the mutex is unlocked and can be acquired.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int lock();

	/**
	*	@memo Description:
	*		Unlock a mutex.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int unlock();
};

#endif		/* MUTEX_H */
