/********************************************************************

	Filename	: TCPServer.h
	Description	: TCP Server Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20010809	Michael Chen: Allow Setting Maximum Clients
	20011120	Michael Chen: Added readClientInfoTagConfig
	20020215	Michael Chen: Added Client Feature Flag

********************************************************************/

#ifndef TCPSERVER_H
#define TCPSERVER_H

#include <poll.h>
#include "TCPSocket.h"

#define DEFAULT_TCPSERVER_CONFIG	"tcpserver.dat"

#define SERVER_CHECK_CLIENT_IP		1

class TCPServer {
protected:
	/***
	*	constant values
	*/
	enum {
		maximum_clients = 20,
		maximum_connection = maximum_clients + 1,
		maxIPlen = 20,
		maxFeature = 40
	};

	/**
	*	@memo Description:
	*		Maximum number of clients set (default 20)
	*/
	int		maxClients;

	/**
	*	@memo Description:
	*		Maximum number of sockets (maxClients + 1, default 21)
	*/
	int		maxConnection;

	/**
	*	@memo Description:
	*		TCP Socket array
	*/
	TCPSocket	tcpsock[maximum_connection];

	/**
	*	@memo Description:
	*		Pointer to listener socket
	*/
	TCPSocket	*server_tcpsock;

	/**
	*	@memo Description:
	*		Pointer to client socket array
	*/
	TCPSocket	*client_tcpsock;

private:
	char		client_ip[maximum_clients][maxIPlen+1];
	unsigned short	client_port[maximum_clients];
	char		client_type[maximum_clients][maxFeature+1];

	int		option_flag;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	TCPServer();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~TCPServer();

	/**
	*	@memo Description:
	*		Read client configuration file "tcpserver.dat" without any tag name,
	*		formatted by each line having 4 fields (client id, client ip, client port, client type),
	*		where client port and client type are both optional fields.
	*		Maximum length of client type is 40 characters.
	*		Client port is 0 or optional means allow any client port.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error
	*	@return
	*		Self reference
	*/
	virtual TCPServer& readClientConfig();

	/**
	*	@memo Description:
	*		Read client configuration file without any tag name,
	*		formatted by each line having 4 fields (client id, client ip, client port, client type),
	*		where client port and client type are both optional fields.
	*		Maximum length of client type is 40 characters.
	*		Client port is 0 or optional means allow any client port.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*/
	virtual TCPServer& readClientConfig(char *filename);

	/**
	*	@memo Description:
	*		Read client configuration file under a tag "[CLIENT_INFO]",
	*		formatted by each line having 4 fields (client id, client ip, client port, client type),
	*		where client port and client type are both optional fields.
	*		Maximum length of client type is 40 characters.
	*		Client port is 0 or optional means allow any client port.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error or tag not found
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*/
	virtual TCPServer& readClientInfoTagConfig(char *filename);

	/**
	*	@memo Description:
	*		Read client configuration file under a dynamic tag, e.g. tag "ABC" means under tag "[ABC]" in the file,
	*		content formatted by each line having 4 fields (client id, client ip, client port, client type),
	*		where client port and client type are both optional fields.
	*		Maximum length of client type is 40 characters.
	*		Client port is 0 or optional means allow any client port.
	*		It can be overrided for different configuration file format.
	*	@exception
	*		GeneralErrorException - open file error or tag not found
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*	@param tag_name (input)
	*		Tag name
	*/
	virtual TCPServer& readClientInfoTagConfig(char *filename, char *tag_name);

	/**
	*	@memo Description:
	*		Set maximum number of client connection.
	*		After that, the server can accept maximum number of connection not exceeding this value.
	*	@exception
	*		Nil
	*	@return
	*		Maximum number of clients
	*	@param max_value (input)
	*		Maximum number of client connection
	*/
	int setMaximumClients(int max_value);

	/**
	*	@memo Description:
	*		Set client configuration for one client ID,
	*		with 3 fields (client id, client ip, client port),
	*		and client type will be empty string.
	*		Client port is 0 or optional means allow any client port.
	*	@exception
	*		Nil
	*	@return
	*		0 if valid client id,
	*		-1 otherwise.
	*	@param id (input)
	*		Client ID
	*	@param c_ip (input)
	*		Client IP
	*	@param c_port (input)
	*		Client port
	*/
	int setClientConfig(int id, char *c_ip, unsigned short c_port);

	/**
	*	@memo Description:
	*		Set client configuration for one client ID,
	*		with 4 fields (client id, client ip, client port, client type),
	*		Maximum length of client type is 40 characters.
	*		Client port is 0 or optional means allow any client port.
	*	@exception
	*		Nil
	*	@return
	*		0 if valid client id,
	*		-1 otherwise.
	*	@param id (input)
	*		Client ID
	*	@param c_ip (input)
	*		Client IP
	*	@param c_port (input)
	*		Client port
	*	@param c_type (input)
	*		Client type
	*/
	int setClientConfig(int id, char *c_ip, unsigned short c_port, char *c_type);

	/**
	*	@memo Description:
	*		Clear client configuration for one client ID,
	*	@exception
	*		Nil
	*	@return
	*		0 if valid client id,
	*		-1 otherwise.
	*	@param id (input)
	*		Client ID
	*/
	int clearClientConfig(int id);

	/**
	*	@memo Description:
	*		Initialize server's listener (create socket, bind, listen)
	*	@exception
	*		GeneralErrorException - bind socket error or listen failure
	*	@return
	*		Self reference
	*	@param port (input)
	*		Listen port
	*	@param flag (input)
	*		Set flag to SERVER_CHECK_CLIENT_IP (1) if client's IP is checked upon connection.
	*		Set flag to 0 if client's IP checking is not necessary.
	*/
	TCPServer& initServer(unsigned short port, int flag);

	/**
	*	@memo Description:
	*		Check if the client ID's connection is established
	*	@exception
	*		Nil
	*	@return
	*		0 if invalid client id or connection not established (disconnected),
	*		1 if valid client id and connection established (connected).
	*	@param id (input)
	*		Client ID
	*/
	int isValidClientConnection(int id);

	/**
	*	@memo Description:
	*		Get number of established connections
	*	@exception
	*		Nil
	*	@return
	*		Number of established connections
	*/
	int getNumActiveClients();

	/**
	*	@memo Description:
	*		Get client IP
	*	@exception
	*		Nil
	*	@return
	*		0 if valid client ID,
	*		-1 otherwise.
	*	@param id (input)
	*		Client ID
	*	@param ip (output)
	*		Client IP to be obtained, empty string if client not connected
	*/
	int getClientIP(int id, char *ip);

	/**
	*	@memo Description:
	*		Get client port
	*	@exception
	*		Nil
	*	@return
	*		Client port, 0 if client not connected
	*	@param id (input)
	*		Client ID
	*/
	unsigned short getClientPort(int id);

	/**
	*	@memo Description:
	*		Get clietn type of a clietn ID
	*	@exception
	*		Nil
	*	@return
	*		0 if valid clietn id,
	*		-1 otherwise.
	*	@param id (input)
	*		Client ID
	*	@param c_type (output)
	*		Client type to be obtained
	*/
	int getClientType(int id, char *c_type);

	/**
	*	@memo Description:
	*		Poll all client event (e.g. incoming data, connection or disconection).
	*		When there is incoming data or disconnection event on a client ID,
	*		virtual function processClientEvent() will be called automatically.
	*		When there is connection request event on a client ID,
	*		virtual function acceptClient() will be called automatically.
	*		The function processClientEvent() should handle the event.
	*		If error detected, processClientEvent() should return -1 and pollClientEvent() will perform disconnection automatically.
	*	@exception
	*		Nil
	*	@return
	*		0 if polling timeout,
	*		-1 if polling error,
	*		otherwise, return number of events.
	*	@param timeout (input)
	*		Timeout value in milli-seconds.
	*/
	virtual int pollClientEvent(int timeout);

	/**
	*	@memo Description:
	*		Accept and make a new client.
	*		This function is called when there is a connection event from client.
	*		This function check if the client connection is valid and
	*		this function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		Self reference
	*/
	virtual TCPServer& makeNewClient();

	/**
	*	@memo Description:
	*		Close the listener.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		Self reference
	*/
	virtual TCPServer& closeServer();

	/**
	*	@memo Description:
	*		Close all active client connections.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		Always 0.
	*/
	virtual int disconnectAllClients();

	/**
	*	@memo Description:
	*		Close a client connection.
	*		This function can be overrided.
	*	@exception
	*		Nil
	*	@return
	*		0 if successfully disconnected,
	*		-1 if client connection already disconnected.
	*	@param id (input)
	*		Client ID
	*/
	virtual int disconnectClient(int id);

	/**
	*	@memo Description:
	*		Accept a client connection
	*		The function can overrided and should be called automatically when a client client is accepted.
	*		The return value is not checked by calling function.
	*	@exception
	*		Nil
	*	@return
	*		Always 0
	*	@param id (input)
	*		Client ID
	*/
	virtual int acceptClient(int id);

	/**
	*       @memo Description:
	*               Process client event, e.g. incoming message or disconnection from client.
	*               It is called automatically when there is a client event when pollClientEvent() is called.
	*               It must be overrided and user should call readClient() to read message
	*               and return -1 when readClient() returns <= 0.
	*       @exception
	*               None
	*       @return
	*               -1 when there is error in event handling, e.g. disconnection case,
	*               0 when successful
	*       @param id (input)
	*               Client ID
	*/
	virtual int processClientEvent(int id) = 0;

	/**
	*	@memo Description:
	*		Read data from a connection
	*	@exception
	*		GeneralErrorException - invalid client ID
	*	@return
	*		0 if disconnected by remote side.
	*		-1 if error,
	*		otherwise, return number of bytes read.
	*	@param id (input)
	*		Client ID
	*	@param buffer (output)
	*		Buffer of data to be read
	*	@param len (input)
	*		Maximum length of buffer can be read
	*/
	int readClient(int id, unsigned char *buffer, int len);

	/**
	*	@memo Description:
	*		Send data to a connection
	*	@exception
	*		GeneralErrorException - invalid client ID
	*	@return
	*		0 if disconnected by remote side.
	*		-1 if error,
	*		otherwise, return number of bytes sent.
	*	@param id (input)
	*		Client ID
	*	@param buffer (output)
	*		Buffer of data to be sent
	*	@param len (input)
	*		Length of buffer to be sent
	*/
	int sendClient(int id, unsigned char *buffer, int len);
};

#endif /* TCPSERVER_H */
