/********************************************************/
/*							*/
/*	Inter-module Communication Message Type		*/
/*	Last Edit: 20010519				*/
/*	Version  : 1.00					*/
/*							*/
/********************************************************/

#ifndef INTM_MESSAGE_H
#define INTM_MESSAGE_H


/*	Maximun message size and segment length		*/
#define MAX_DATA	65516
#define MAX_SEG_LEN	255

#define HEADERSIZE	14

/*****
typedef struct {
	long	mtype;
	unsigned char	mtext[MAX_DATA+14];
} super_buffer;
*****/

typedef struct {
	unsigned param_len;			/* Parameter Length 		*/
	unsigned char param_iden;		/* Parameter Identifier		*/
	unsigned char reserved[3];		/* Reserved */
	unsigned char param_byte[MAX_DATA];	/* Parameter Data Byte 		*/
} param_segment2;


struct intm_msg {
	/**
	*	@memo Description:
	*		Length indicator
	*/
	unsigned short len_indicator;

	/**
	*	@memo Description:
	*		Source point code
	*/
	unsigned char SPC;

	/**
	*	@memo Description:
	*		Destination point code
	*/
	unsigned char DPC;

	/**
	*	@memo Description:
	*		Sub-address of the source point
	*/
	unsigned short SUBSP;

	/**
	*	@memo Description:
	*		Sub-address of the destination point
	*/
	unsigned short SUBDP;

	/**
	*	@memo Description:
	*		Message type
	*/
	unsigned short mess_type;

	/**
	*	@memo Description:
	*		Sequence number
	*/
	unsigned short seq_num;

	/**
	*	@memo Description:
	*		Reserved for future use
	*/
	unsigned char reserved;

	/**
	*	@memo Description:
	*		Parameter segment count
	*/
	unsigned char param_count;

	/**
	*	@memo Description:
	*		Parameter segment
	*/
	unsigned char data[MAX_DATA];

	/**
	*	@memo Description:
	*		Initialize intm_msg header
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param spc (input)
	*		source point code
	*	@param dpc (input)
	*		destination point code
	*	@param subsp (input)
	*		sub-source point code
	*	@param subdp (input)
	*		sub-destination point code
	*	@param seq_number (input)
	*		sequence number
	*	@param message_type (input)
	*		message type.  If 0, whole message will reset
	*/
	void init_msg(unsigned char spc, unsigned char dpc,
		unsigned short subsp, unsigned short subdp,
		unsigned short seq_number, unsigned short message_type);

	/**
	*	@memo Description:
	*		Modify intm_msg header
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param spc (input)
	*		source point code
	*	@param dpc (input)
	*		destination point code
	*	@param subsp (input)
	*		sub-source point code
	*	@param subdp (input)
	*		sub-destination point code
	*	@param seq_number (input)
	*		sequence number
	*/
	void reinit_msg(unsigned char spc, unsigned char dpc,
		unsigned short subsp, unsigned short subdp,
		unsigned short seq_number);

	/**
	*	@memo Description:
	*		Add parameter to intm_msg
	*	@exception
	*		None
	*	@return
	*		-1 when invalid input parameter ID or length,
	*		else return length of whole message without header
	*	@param param_id (input)
	*		parameter ID
	*	@param param_byte (input)
	*		parameter content
	*	@param length (input)
	*		length of the parameter content
	*/
	int add_param(unsigned char param_id, const unsigned char *param_byte, int length);

	/**
	*	@memo Description:
	*		Get parameter to intm_msg from the beginning (without modification of the message)
	*	@exception
	*		None
	*	@return
	*		0 if parameter not found,
	*		else return length of the parameter found
	*		(include parameter indicator and parameter length portion)
	*	@param param_id (input)
	*		parameter ID
	*	@param param_byte (output)
	*		parameter content to be obtained
	*	@param length (output)
	*		length of the parameter content to be obtained
	*/
	int get_param(unsigned char param_id, unsigned char *param_byte, int *length);

	/**
	*	@memo Description:
	*		Get and remove parameter from intm_msg from the beginning.
	*		After calling this function, do not call read_param().
	*	@exception
	*		None
	*	@return
	*		0 if parameter not found,
	*		else return length of the parameter found
	*		(include parameter indicator and parameter length portion)
	*	@param param_id (input)
	*		parameter ID
	*	@param param_byte (output)
	*		parameter content to be obtained
	*	@param length (output)
	*		length of the parameter content to be obtained
	*/
	int del_param(unsigned char param_id, unsigned char *param_byte, int *length);

	/**
	*	@memo Description:
	*		Check if parameter exists in the intm_msg and obtain the parameter (without modification of the message)
	*	@exception
	*		None
	*	@return
	*		0 if parameter not found,
	*		else return length of the parameter found
	*		(include parameter indicator and parameter length portion)
	*	@param param_id (input)
	*		parameter ID
	*	@param param_seg (output)
	*		parameter segment to be obtained
	*/
	int contain_param(unsigned char param_id, unsigned char *param_seg);

	/**
	*	@memo Description:
	*		Get the message type of intm_msg and convert it into string format.
	*		It is multi-thread safe only if 'out_str' is not null
	*	@exception
	*		None
	*	@return
	*		NULL if intm_msg is invalid,
	*		otherwise, return character pointer of output string
	*	@param out_str (output)
	*		output string buffer.  If NULL, internal memory will be used
	*/
	char *print_intm_msg_type(char *out_str);

	/**
	*	@memo Description:
	*		Print the details of intm_msg
	*	@exception
	*		None
	*	@return
	*		0
	*/
	int print_intm_msg_all();

};


class intm_reader{
	/** Pointer to original message **/
	intm_msg *p_msg;
	/** Running pointer **/
	unsigned short seg_pt;
	/** Running param counter **/
	unsigned char param_count;

public:
	/**
	*	@memo Description:
	*		Constructor
	*	@param p (input)
	*		intm_msg to reference
	*/
	intm_reader(intm_msg *p = NULL);

	/**
	*	@memo Description:
	*		Read each parameter from intm_msg
	*	@exception
	*		None
	*	@return
	*		0 if no more parameter can be read,
	*		-1 if message content corrupted,
	*		successful otherwise
	*	@param seg_message (output)
	*		parameter segment to be read
	*/
	int read_param(param_segment2 *seg_message);
};

#endif
