/********************************************************************

	Filename	: RW_Lock.h
	Description	: Read Write Lock Class header file

	History:
	------------------------------------------------------------
	20011206	Michael Chen: First Creation

********************************************************************/

#ifndef RW_LOCK_H
#define RW_LOCK_H

#ifndef __linux__

#include <synch.h>

class RW_Lock {
private:
	rwlock_t	*rwlp;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	RW_Lock();

	/**
	*	@memo Description:
	*		Constructor with read-write lock memory initialization
	*	@param ptr (input)
	*		Read-write lock pointer to be initialized
	*/
	RW_Lock(rwlock_t *ptr);

	/**
	*	@memo Description:
	*		Default destructor (without destroying the read-write lock)
	*/
	~RW_Lock();

	/**
	*	@memo Description:
	*		Initialize read-write lock memory by assigning the read-write lock address
	*	@exception
	*		None
	*	@return
	*		0 if valid pointer,
	*		-1 otherwise
	*	@param ptr (input)
	*		Read-write lock pointer to be initialized
	*/
	int init_mem(rwlock_t *ptr);

	/**
	*	@memo Description:
	*		Initialize read-write lock memory by creating read-write lock in local memory
	*	@exception
	*		None
	*	@return
	*		0
	*/
	int init_mem();

	/**
	*	@memo Description:
	*		Initialize a read-write lock after initialized read-write lock memory with types
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*	@param type (input)
	*		Type of read-write lock.
	*		USYNC_THREAD for synchronization among threads only.
	*		USYNC_PROCESS for synchronization among threads and processes.
	*/
	int init(int type);

	/**
	*	@memo Description:
	*		Initialize a read-write lock with USYNC_PROCESS type
	*		after initialized read-write lock memory
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int init();

	/**
	*	@memo Description:
	*		Destroy a read-write lock
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int destroy();

	/**
	*	@memo Description:
	*		Try to read-lock a read-write lock.
	*		Function return immediately no matter the read-write lock can be locked successfully or not
	*	@exception
	*		None
	*	@return
	*		0 if successfully read-locked a read-write lock,
	*		EBUSY if the read-write lock is already write-locked by another process / thread,
	*		errno otherwise
	*/
	int tryreadlock();

	/**
	*	@memo Description:
	*		Try to write-lock a read-write lock.
	*		Function return immediately no matter the read-write lock can be locked successfully or not
	*	@exception
	*		None
	*	@return
	*		0 if successfully write-locked a read-write lock,
	*		EBUSY if the read-write lock is already read-locked or write-locked by another process / thread,
	*		errno otherwise
	*/
	int trywritelock();

	/**
	*	@memo Description:
	*		Try to lock (write-lock) a read-write lock.  (Same as trywritelock.)
	*		Function return immediately no matter the read-write lock can be locked successfully or not
	*	@exception
	*		None
	*	@return
	*		0 if successfully locked (write-locked) a read-write lock,
	*		EBUSY if the read-write lock is already read-locked or write-locked by another process / thread,
	*		errno otherwise
	*/
	int trylock();

	/**
	*	@memo Description:
	*		Read-lock a read-write lock.
	*		If the read-write lock is being write-locked by another process,
	*		it will wait until the read-write lock is unlocked and can be acquired.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int readlock();

	/**
	*	@memo Description:
	*		Write-lock a read-write lock.
	*		If the read-write lock is being read-locked or write-locked by another process,
	*		it will wait until the read-write lock is unlocked and can be acquired.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int writelock();

	/**
	*	@memo Description:
	*		Lock (write-lock) a read-write lock.  (Same as writelock.)
	*		If the read-write lock is being read-locked or write-locked by another process,
	*		it will wait until the read-write lock is unlocked and can be acquired.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int lock();

	/**
	*	@memo Description:
	*		Unlock a read-write lock.
	*	@exception
	*		None
	*	@return
	*		0 if successful,
	*		errno otherwise
	*/
	int unlock();
};

#endif		/* __linux__ */

#endif		/* RW_LOCK_H */
