/********************************************************************

	Filename	: SysParam.h
	Description	: System Parameter header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011206	Michael Chen: Support both Shared Mem / Non-Shared Mem

********************************************************************/

#ifndef SYS_PARAM_H
#define SYS_PARAM_H

#include "IPC_Memory.h"

#define DEFAULT_SYS_PARAM_CONFIG	"sysparam.dat"

class SysParam {
public:
	/***
	*	constant values
	*/
	enum {
		numParamValue = 32,
		numParamString = 32,
		maxParamStringLen = 32,
		maxConfigFilenameLen = 32
	};

private:
	IPC_Memory	*shm_mem;
	char		config_file[maxConfigFilenameLen + 1];

	typedef struct {
		long		param_val[numParamValue];
		unsigned char	param_str[numParamString][maxParamStringLen+1];
	} sys_param_s;

	sys_param_s	*param_data;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	SysParam();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~SysParam();

	/**
	*	@memo Description:
	*		Initialize shared memory for storing system parameters
	*	@exception
	*		GeneralErrorException - shared memory initialization error
	*	@return
	*		Self reference
	*	@param key (input)
	*		Shared memory key
	*	@param flag (input)
	*		Shared memory initialization flag, e.g. IPC_CREAT, PERMS
	*/
	SysParam& init(int key, int flag);

	/**
	*	@memo Description:
	*		Initialize local memory for storing system parameters
	*	@exception
	*		None
	*	@return
	*		Self reference
	*/
	SysParam& init();

	/**
	*	@memo Description:
	*		Check if the system parameter memory is already initialized
	*	@exception
	*		None
	*	@return
	*		1 if valid,
	*		0 if not valid
	*/
	int isValid();

	/**
	*	@memo Description:
	*		Reload system parameters from configuration file into memory.
	*		The configuration file name is defaultly using "sysparam.dat"
	*	@exception
	*		None
	*	@return
	*		Self reference
	*/
	SysParam& reloadConfig();

	/**
	*	@memo Description:
	*		Reload system parameters from configuration file into memory with filename specified.
	*	@exception
	*		None
	*	@return
	*		Self reference
	*	@param filename (input)
	*		Configuration file name
	*/
	SysParam& reloadConfig(const char *filename);

	/**
	*	@memo Description:
	*		View and print system parameters to standand output
	*	@exception
	*		None
	*	@return
	*		Self reference
	*/
	SysParam& viewConfig();

	/**
	*	@memo Description:
	*		View and print system parameters to a given file pointer
	*	@exception
	*		None
	*	@return
	*		Self reference
	*	@param f (input)
	*		File pointer where the system parameters to be printed to
	*/
	SysParam& viewConfig(FILE *f);

	/**
	*	@memo Description:
	*		Get parameter value
	*	@exception
	*		GeneralErrorException - memory not initialized
	*	@return
	*		Parameter value
	*	@param id (input)
	*		Parameter ID.  Valid value from 0 to 31.
	*/
	long getParamValue(int id);

	/**
	*	@memo Description:
	*		Get parameter string
	*	@exception
	*		GeneralErrorException - memory not initialized
	*	@return
	*		Self reference
	*	@param id (input)
	*		Parameter ID.  Valid value from 0 to 31.
	*	@param string (output)
	*		Parameter string to be obtained
	*/
	SysParam& getParamString(int id, char *string);
};

#endif		/* SYS_PARAM_H */
