/********************************************************************

	Filename	: TCPSocket.h
	Description	: TCP Socket Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20020415	Ho Yuen Kei: Add Thread for connection
	20020601	Michael Chen: Option for connectSocket() and connectSocketThread()

********************************************************************/

#ifndef TCPSOCKET_H
#define TCPSOCKET_H

#ifdef WIN32
#include <windows.h>
#elif defined(__linux__)
#include <pthread.h>
#else
#include <thread.h>
#endif

#define SOCKET_STATUS_IDLE		0
#define SOCKET_STATUS_CONNECTING	1

class TCPSocket {
public:
	struct socket_connection_struct {
		/***
		*	@memo Description
		*		File descriptor
		*/
		int		fd;

		/***
		*	@memo Description
		*		Connection status
		*/
		int		status;

		/***
		*	@memo Description
		*		Server IP
		*/
		char		ip[512];

		/***
		*	@memo Description
		*		Server port
		*/
		unsigned short	port;

#ifdef WIN32
		/***
		*	@memo Description
		*		Connection thread ID
		*/
		DWORD	connect_tid;
#elif defined(__linux__)
		/***
		*	@memo Description
		*		Connection thread ID
		*/
		pthread_t	connect_tid;
#else
		/***
		*	@memo Description
		*		Connection thread ID (POSIX)
		*/
		thread_t	connect_tid;
#endif
	};

private:
	socket_connection_struct	socket_data;

public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	TCPSocket();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~TCPSocket();

	/**
	*	@memo Description:
	*		Initialize socket file descriptor of a created socket
	*	@exception
	*		GeneralErrorException - invalid file descriptor or error in socket state
	*	@return
	*		Self reference
	*	@param in_fd (input)
	*		Socket file descriptor
	*/
	TCPSocket& setsockfd(int in_fd);

	/**
	*	@memo Description:
	*		Get socket file descriptor
	*	@exception
	*		None
	*	@return
	*		Socket file descriptor
	*/
	int getsockfd();

	/**
	*	@memo Description:
	*		Create a new socket
	*	@exception
	*		GeneralErrorException - socket already created or create socket failure
	*	@return
	*		Self reference
	*/
	TCPSocket& createSocket();

	/**
	*	@memo Description:
	*		Close socket
	*	@exception
	*		Nil
	*	@return
	*		Self reference
	*/
	TCPSocket& closeSocket();

	/**
	*	@memo Description:
	*		Set socket reuse option
	*	@exception
	*		GeneralErrorException - set socket option failure
	*	@return
	*		Self reference
	*/
	TCPSocket& setSocketOptionReuseOn();

	/**
	*	@memo Description:
	*		Set socket unblock
	*	@exception
	*		GeneralErrorException - set socket unblock failure
	*	@return
	*		Self reference
	*/
	TCPSocket& setSocketUnblock();

	/**
	*	@memo Description:
	*		Bind a port
	*	@exception
	*		GeneralErrorException - bind failure
	*	@return
	*		Self reference
	*	@param port (input)
	*		Port number
	*/
	TCPSocket& bindSocket(unsigned short port);

	/**
	*	@memo Description:
	*		Start listener of the socket
	*	@exception
	*		GeneralErrorException - listen failure
	*	@return
	*		Self reference
	*	@param maxClients (input)
	*		Maximum client connection request
	*/
	TCPSocket& listenSocket(int maxClients);

	/**
	*	@memo Description:
	*		Accept a connection
	*	@exception
	*		GeneralErrorException - accept failure
	*	@return
	*		Accepted socket connection file descriptor
	*	@param ip (output)
	*		Remote connection IP
	*	@param port (output)
	*		Remote connection port
	*/
	int acceptNewSocket(char *ip, unsigned short *port);

	/**
	*	@memo Description:
	*		Connect to a listener
	*		(blocking, return when connect successful or timeout)
	*	@exception
	*		GeneralErrorException - connect failure
	*	@return
	*		Self reference
	*	@param ip (input)
	*		Remote connection IP
	*	@param port (input)
	*		Remote connection port
	*/
	TCPSocket& connectSocket(const char *ip, unsigned short port);

	/**
	*	@memo Description:
	*		Connect to a listener using thread
	*		(non-blocking, return immediately whenever connect successful or not).
	*		User need to check whether the connection is valid by isValid()
	*		before sending or receiving data.
	*	@exception
	*		GeneralErrorException - connect failure (e.g. thread creation error)
	*	@return
	*		Self reference
	*	@param ip (input)
	*		Remote connection IP
	*	@param port (input)
	*		Remote connection port
	*/
	TCPSocket& connectSocketThread(const char *ip, unsigned short port);

	/**
	*	@memo Description:
	*		Check if the socket is valid
	*	@exception
	*		None
	*	@return
	*		1 if valid,
	*		0 otherwise
	*/
	int isValid();
};

#endif /* TCPSOCKET_H */
